@@ -19,27 +19,107 @@ class WKFSAdapter:
         logging=False,
         wkfs_config=None,
         template_name=None,
+        timeout=None
     ):
-        self._api_handler = APIHandler(host, headers={}, logging=logging)
+        DEFAULT_TIMEOUT = 60
+        self._api_handler = APIHandler(host, headers={}, logging=logging, timeout=timeout if timeout else DEFAULT_TIMEOUT)
         self.template_name = template_name
 
         self.wkfs_config = wkfs_config
         self.host = host
         self.auth_host = auth_host
 
+    def begin_docviewer_session(self, 
+            transaction_xml_payload: str,
+            callback_url: str = None,
+            package_name: str = None,
+            log_config: dict = None,
+            access_token: str = None):
+        docviewer_payload = self._generate_docviewer_payload(transaction_xml_payload, callback_url, package_name)
+        headers = self._api_handler._headers
+        headers["Authorization"] = f"Bearer {access_token}"
+        headers["Content-Type"] = "application/json"
+        self._api_handler._host = self.host
+        response = self._api_handler.send_request(
+            "POST",
+            f"/DGSCoreDocViewer/api/Sessions/V2/BeginSession",
+            payload=json.dumps(docviewer_payload),
+            log_config=log_config,
+            headers=headers
+        )
+
+        LOGGER.debug(f"begin_docviewer_session from wkfs wrapper completed")
+        return json.loads(response)
+    
+    def get_docviewer_session(self, session_id, 
+            access_token: str = None,
+            log_config: dict = None):
+        headers = self._api_handler._headers
+        headers["Authorization"] = f"Bearer {access_token}"
+        headers["Content-Type"] = "application/json"
+        self._api_handler._host = self.host
+        response = self._api_handler.send_request(
+            "GET",
+            f"/DGSCoreDocViewer/api/Sessions/{session_id}",
+            log_config=log_config,
+            payload=json.dumps({"Test":"Test"}),
+            headers=headers
+        )
+
+        LOGGER.debug(f"get_docviewer_session from wkfs wrapper completed")
+        return json.loads(response)
+
+    def _generate_docviewer_payload(self, transaction_xml_payload, callback_url, package_name):
+        payload = {}
+        account = {}
+        wkfs_id = self.wkfs_config["wkfs_id"]
+        packages = self.wkfs_config["packages"]
+        account_id = self.wkfs_config["account_id"]
+        options = self.wkfs_config["docviewer_options"]
+        account["id"] = int(account_id)
+        payload["account"] = account
+        cleaned_transaction_xml_payload = clean_transaction_xml(transaction_xml_payload)
+
+        transaction_xml_payload_bytes = cleaned_transaction_xml_payload
+
+        base64_bytes = base64.b64encode(transaction_xml_payload_bytes)
+        transaction_data_base64 = base64_bytes.decode("utf-8")
+        payload["transactionData"] = transaction_data_base64
+        payload["callbackPath"] = callback_url
+ 
+        wkfs_package_name = None
+        
+        for package in packages:
+            config_package_name = package.get("name")
+
+            if package_name == config_package_name:
+                wkfs_package_name = package.get("wkfs_package_name")
+                break
+        packages = []
+        package = {}
+        package["id"] = f"expere://{wkfs_id}/{wkfs_package_name}"
+        package["name"] = wkfs_package_name
+        packages.append(package)
+        payload["options"] = options
+        payload["packages"] = packages
+        return payload
+
     def generate_package(
         self,
-        transaction_data_json_input: str,
+        transaction_data_json_input: str = None,
+        transaction_data_xml_input: str = None,
         e_sign: bool = False,
         package_name: str = None,
         log_config: dict = None,
         access_token: str = None,
+        interface_config: dict = None,
     ) -> dict:
         """
         Call the `send` API for generating the document.
 
         :param
             transaction_data_json_input: Json input from the calling application to generate the transaction xml
+            transaction_data_xml_input: Xml input from the calling application to generate the transaction xml
             e_sign: Indicating whether e signature co-ordinates should be part of response
             package_name: The package for which documents are generated.
             access_token: The access token required to authenticate the caller.
@@ -68,6 +148,7 @@ class WKFSAdapter:
         account_id = self.wkfs_config["account_id"]
         ezConfig = self.wkfs_config["ezConfig"]
 
+        wkfs_package = None
         wkfs_package_name = None
         wkfs_xml = None
         for package in packages:
@@ -103,17 +184,18 @@ class WKFSAdapter:
 
         template = env.get_template(wkfs_xml)
 
-        data_dict = json.loads(transaction_data_json_input)
-        transaction_xml_payload = template.render(**data_dict)
+        if transaction_data_json_input:
+            data_dict = json.loads(transaction_data_json_input)
+            transaction_xml_payload = template.render(**data_dict)
 
-        cleaned_transaction_xml_payload = clean_transaction_xml(transaction_xml_payload)
+            cleaned_transaction_xml_payload_bytes = clean_transaction_xml(transaction_xml_payload)
+            
+            # file1 = open("./clean_final_code.xml", "wb")
+            # file1.write(cleaned_transaction_xml_payload)
+        else:
+            cleaned_transaction_xml_payload_bytes = transaction_data_xml_input.encode('utf-8')
 
-        # file1 = open("./clean_final_code.xml", "wb")
-        # file1.write(cleaned_transaction_xml_payload)
-
-        transaction_xml_payload_bytes = cleaned_transaction_xml_payload
-
-        base64_bytes = base64.b64encode(transaction_xml_payload_bytes)
+        base64_bytes = base64.b64encode(cleaned_transaction_xml_payload_bytes)
         transaction_data_base64 = base64_bytes.decode("utf-8")
 
         request["transactionData"] = transaction_data_base64
@@ -123,6 +205,18 @@ class WKFSAdapter:
         generate["request"] = request
         payload["generate"] = generate
 
+        if interface_config:
+            interface_details = {}
+            interface_details["transaction_xml"] = cleaned_transaction_xml_payload_bytes.decode("utf-8").replace("\\n", "").replace("\\t", "")
+            interface_details["updated_xml"] = None
+            interface_details["changed_data_points"] = None
+            interface_entry = interface_config["model"](
+                loan=interface_config["loan"],
+                interface_type=interface_config["interface_type"],
+                details = json.dumps(interface_details)
+            )
+            interface_entry.save()
+
         headers = self._api_handler._headers
         headers["Authorization"] = f"Bearer {access_token}"
         headers["Content-Type"] = "application/json"
@@ -133,7 +227,7 @@ class WKFSAdapter:
             f"/DocumentService/api/v1/Document/account/{account_id}/generate-synchronous",
             payload=json.dumps(payload),
             log_config=log_config,
-            headers=headers,
+            headers=headers
         )
 
         LOGGER.debug(f"generate_package from wkfs wrapper completed")
